
#################################################
# HolAdo (Holistic Automation do)
#
# (C) Copyright 2021-2025 by Eric Klumpp
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

# The Software is provided “as is”, without warranty of any kind, express or implied, including but not limited to the warranties of merchantability, fitness for a particular purpose and noninfringement. In no event shall the authors or copyright holders be liable for any claim, damages or other liability, whether in an action of contract, tort or otherwise, arising from, out of or in connection with the software or the use or other dealings in the Software.
#################################################

import logging
from holado.common.handlers.object import DeleteableObject, Object
from holado_core.common.tools.tools import Tools
from holado_core.common.exceptions.element_exception import ElementNotFoundException
from holado_python.standard_library.typing import Typing

logger = None

def initialize_logger():
    global logger
    logger = logging.getLogger(__name__)


class Context(DeleteableObject):
    """
    @summary: Mother class for any context class
    """
    
    def __init__(self, name):
        super().__init__(name)
        self.on_delete_call_gc_collect = True
        
        self.__objects = {}
        self.__on_delete_objects = []
        
    def get_context_name(self):
        return Object.name(self)
    
    def _delete_object(self):
        # Remove all objects
        self.remove_all_objects()
    
    def remove_all_objects(self):
        from holado_core.common.exceptions.functional_exception import FunctionalException
        from holado_core.common.exceptions.technical_exception import TechnicalException
        
        keys = set(self.__objects.keys())
        if Tools.do_log(logger, logging.DEBUG):
            logger.debug(f"[{self.name}] Removing {len(keys)} objects: {keys}")
        for index, key in enumerate(keys):
            if Tools.do_log(logger, logging.DEBUG):
                logger.debug(f"[{self.name}] Removing object {index+1}/{len(keys)}: '{key}' (type: {Typing.get_object_class_fullname(self.get_object(key))})")
            try:
                self.remove_object(key)
            except FunctionalException as exc:
                raise TechnicalException() from exc
        if Tools.do_log(logger, logging.DEBUG):
            logger.debug(f"[{self.name}] Finished to remove {len(keys)} objects: {keys}")
                
    def remove_object(self, name):
        if not self.has_object(name):
            from holado_core.common.exceptions.technical_exception import TechnicalException
            raise TechnicalException("Context doesn't contain object '{}'".format(name))
        self._remove(name, True)
        
    def _remove(self, name, remove_from_lifetime_manager):
        obj = self.get_object(name)
        if Tools.do_log(logger, logging.DEBUG):
            logger.debug(f"[{self.name}] Removing object '{name}' (type: {Typing.get_object_class_fullname(obj)})")
        if name in self.__on_delete_objects:
            if isinstance(obj, DeleteableObject):
                if Tools.do_log(logger, logging.TRACE):  # @UndefinedVariable
                    logger.trace(f"[{self.name}] Deleting object '{name}'")
                try:
                    obj.delete_object()
                except Exception:
                    logger.exception(f"[{self.name}] Catched exception while deleting object '{name}'")
                else:
                    if Tools.do_log(logger, logging.DEBUG):
                        logger.debug(f"[{self.name}] Deleted object '{name}'")
            self.__on_delete_objects.remove(name)
            
        if self.has_object(name):
            del self.__objects[name]
#         if remove_from_lifetime_manager:
#             from holado.common.context.session_context import SessionContext
#             SessionContext.instance().get_object_lifetime_manager().remove_object_lifetime(name, self)
        
    def set_object(self, name, obj, raise_if_already_set = True):
        from holado_core.common.exceptions.functional_exception import FunctionalException
        from holado_core.common.exceptions.technical_exception import TechnicalException
        
        if obj is None:
            if self.has_object(name):
                try:
                    self.remove_object(name)
                except FunctionalException as e:
                    raise TechnicalException() from e
        else:
            if raise_if_already_set and self.has_object(name):
                raise TechnicalException(f"[{self.name}] Context already contains object '{name}'")
            self._put(name, obj)
                    
    def _put(self, name, obj):
        self.__objects[name] = obj
        self.__on_delete_objects.append(name)
        
    def get_object(self, name):
        if not self.has_object(name):
            from holado_core.common.exceptions.technical_exception import TechnicalException
            raise TechnicalException(f"[{self.name}] Context doesn't contain object '{name}'")
        return self.__objects[name]

    def has_object(self, name):
        return name in self.__objects
    
    def get_object_name(self, obj):
        for name, value in self.__objects.items():
            if value is obj:
                return name
        return None
        
    def remove_lifetimed_object(self, object_id):
        self._remove(object_id, False)
        
    def get_or_create_initilizable_object_by_class(self, object_class):
        raise NotImplementedError()
        
    def get_or_create_object_by_class(self, object_class):
        raise NotImplementedError()
    
    def get_object_getter_eval_string(self, obj, raise_not_found=True):
        name = self.get_object_name(obj)
        if name is not None:
            return f"{Typing.get_object_class_fullname(self)}.instance().get_object('{name}')"
        
        if raise_not_found:
            raise ElementNotFoundException(f"[{self.name}] Failed to find object of id {id(obj)}")
        else:
            return None






#################################################
# HolAdo (Holistic Automation do)
#
# (C) Copyright 2021-2025 by Eric Klumpp
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

# The Software is provided “as is”, without warranty of any kind, express or implied, including but not limited to the warranties of merchantability, fitness for a particular purpose and noninfringement. In no event shall the authors or copyright holders be liable for any claim, damages or other liability, whether in an action of contract, tort or otherwise, arising from, out of or in connection with the software or the use or other dealings in the Software.
#################################################

from holado_core.common.exceptions.functional_exception import FunctionalException
import time
import logging
from holado_core.common.exceptions.technical_exception import TechnicalException
from holado.common.handlers.object import DeleteableObject
from holado_core.common.tools.tools import Tools

logger = logging.getLogger(__name__)

try:
    import docker
    with_docker = True
except Exception as exc:
    if Tools.do_log(logger, logging.DEBUG):
        logger.debug(f"DockerClient is not available. Initialization failed on error: {exc}")
    with_docker = False


class DockerClient(object):
    @classmethod
    def is_available(cls):
        return with_docker
    
    def __init__(self):
        self.__client = docker.from_env()
        self.__containers = {}
        self.__volumes = {}
        
    @property
    def client(self):
        return self.__client
    
    def has_container(self, name, in_list=True, reset_if_removed=True):
        # Note: Even if name exists in __containers, it is possible that the container has been removed
        if in_list:
            c = self.__get_container_from_list(name)
            res = c is not None
            
            if reset_if_removed and not res and name in self.__containers:
                del self.__containers[name]
        else:
            res = name in self.__containers
            
            if reset_if_removed and res:
                if self.__containers[name].status == "removed":
                    del self.__containers[name]
                res = False
        return res
    
    def get_container(self, name, reset_if_removed=True):
        # Reset container if removed
        if reset_if_removed and name in self.__containers:
            if self.__containers[name].status == "removed":
                del self.__containers[name]
        
        # Get container from list if needed
        if name not in self.__containers:
            c = self.__get_container_from_list(name)
            if c:
                self.__containers[name] = DockerContainer(self, c)
        
        return self.__containers.get(name)
    
    def get_all_container_names(self):
        res = []
        for c in self.__client.containers.list(all=True):
            try:
                c_name = c.name
            except docker.errors.NotFound:
                # Container 'c' doesn't exist anymore
                continue
            
            res.append(c_name)
        return res
    
    def __get_container_from_list(self, name):
        res = None
        for c in self.__client.containers.list(all=True):
            try:
                c_name = c.name
            except docker.errors.NotFound:
                # Container 'c' doesn't exist anymore
                continue
            
            if c_name == name:
                res = c
                break
        return res
    
    def has_volume(self, name, in_list = False):
        res = name in self.__volumes
        if not res and in_list:
            v = self.__get_volume_from_list(name)
            res = v is not None
        return res
    
    def get_volume(self, name):
        if name not in self.__volumes:
            v = self.__get_volume_from_list(name)
            if v:
                self.__volumes[name] = DockerVolume(v)
        return self.__volumes.get(name)
    
    def get_all_volume_names(self):
        return [v.name for v in self.__client.volumes.list()]
    
    def __get_volume_from_list(self, name):
        res = None
        for v in self.__client.volumes.list():
            if v.name == name:
                res = v
                break
        return res
    
    def run_container(self, name, image, remove_existing=False, wait_running=True, auto_stop=True, **kwargs):
        if Tools.do_log(logger, logging.DEBUG):
            logger.debug(f"Running docker container '{name}' with image '{image}' and arguments {kwargs}{', and waiting running status' if wait_running else ''}")
        
        # Manage remove if already existing
        cont = self.get_container(name)
        if cont:
            if remove_existing:
                if cont.status == "running":
                    if Tools.do_log(logger, logging.DEBUG):
                        logger.debug(f"Docker container '{name}' is running, stopping it before remove")
                    self.stop_container(name)
                
                if self.has_container(name):    # After stop, container is able to be automatically removed depending on its run parameters 
                    self.remove_container(name)
            else:
                logger.info(f"Docker container '{name}' is already running")
                return
        
        # Run container
        c = self.__client.containers.run(image, name=name, detach=True, **kwargs)
        container = DockerContainer(self, c)
        self.__containers[name] = container
        
        # Manage wait running status
        if wait_running:
            for _ in range(100):
                time.sleep(1)
                if container.status == "running":
                    break
            if container.status != "running":
                raise TechnicalException("Failed to run container of name '{}' (status: {})".format(name, container.status))
        if Tools.do_log(logger, logging.DEBUG):
            logger.debug(f"Run docker container '{name}' with image '{image}' and arguments {kwargs}{', and wait running status' if wait_running else ''}")
            
        # Set properties
        container.auto_stop = auto_stop
        
        return container
    
    def restart_container(self, name, wait_running=True, **kwargs):
        if Tools.do_log(logger, logging.DEBUG):
            logger.debug(f"Restarting docker container '{name}' with arguments {kwargs}{', and waiting running status' if wait_running else ''}")
        container = self.get_container(name)
        if not container:
            raise FunctionalException("Container of name '{}' doesn't exist")
        
        container.restart(**kwargs)
        
        if wait_running:
            for _ in range(120):
                time.sleep(1)
                if container.status == "running":
                    break
            if container.status != "running":
                raise TechnicalException("Failed to restart container of name '{}' (status: {})".format(name, container.status))
        if Tools.do_log(logger, logging.DEBUG):
            logger.debug(f"Restarted docker container '{name}' with arguments {kwargs}{', and waited running status' if wait_running else ''}")
    
    def start_container(self, name, wait_running=True, **kwargs):
        if Tools.do_log(logger, logging.DEBUG):
            logger.debug(f"Starting docker container '{name}' with arguments {kwargs}{', and waiting running status' if wait_running else ''}")
        container = self.get_container(name)
        if not container:
            raise FunctionalException("Container of name '{}' doesn't exist")
        
        container.start(**kwargs)
        
        if wait_running:
            for _ in range(120):
                time.sleep(1)
                if container.status == "running":
                    break
            if container.status != "running":
                raise TechnicalException("Failed to start container of name '{}' (status: {})".format(name, container.status))
        if Tools.do_log(logger, logging.DEBUG):
            logger.debug(f"Started docker container '{name}' with arguments {kwargs}{', and waited running status' if wait_running else ''}")
        
    def stop_container(self, name):
        if Tools.do_log(logger, logging.DEBUG):
            logger.debug(f"Stopping docker container of name '{name}'")
        if name not in self.__containers:
            raise FunctionalException("Unknown container of name '{}'".format(name))
        elif self.__containers[name].status != "running":
            raise FunctionalException("Container of name '{}' is not running (status: {})".format(name, self.__containers[name].status))
        
        self.__containers[name].stop()
        try:
            self.__containers[name].wait()
        except docker.errors.NotFound:
            # This exception occurs on containers automatically removed on stop
            pass
        
        if self.__containers[name].status == "running":
            raise FunctionalException("Failed to stop container of name '{}' (status: {})".format(name, self.__containers[name].status))
        else:
            if Tools.do_log(logger, logging.DEBUG):
                logger.debug(f"Stopped docker container of name '{name}'")
        
    def remove_container(self, name):
        if Tools.do_log(logger, logging.DEBUG):
            logger.debug(f"Removing docker container of name '{name}'")
        if not self.has_container(name):
            raise FunctionalException(f"Container of name '{name}' doesn't exist")
        
        del self.__containers[name]
        self.client.api.remove_container(name)
        
        if self.has_container(name):
            raise FunctionalException(f"Failed to remove container of name '{name}'")
        else:
            if Tools.do_log(logger, logging.DEBUG):
                logger.debug(f"Removed docker container of name '{name}'")
        
class DockerContainer(DeleteableObject):
    def __init__(self, docker_client, container):
        super().__init__(container.name)
        
        self.__docker_client = docker_client
        self.__container = container
        self.__auto_stop = False
        
    def _delete_object(self):
        if self.auto_stop and self.__docker_client and self.__docker_client.has_container(self.name) \
                and self.__docker_client.get_container(self.name).status == "running":
            self.__docker_client.stop_container(self.name)
        
    @property
    def container(self):
        return self.__container
        
    @property
    def status(self):
        try:
            self.__container.reload()
        except docker.errors.NotFound:
            return "removed"
        return self.__container.status
        
    @property
    def auto_stop(self):
        self.__auto_stop
        
    @auto_stop.setter
    def auto_stop(self, auto_stop):
        self.__auto_stop = auto_stop
    
    def restart(self, **kwargs):
        return self.__container.restart(**kwargs)
    
    def start(self, **kwargs):
        return self.__container.start(**kwargs)
    
    def stop(self, **kwargs):
        return self.__container.stop(**kwargs)
    
    def wait(self, **kwargs):
        return self.__container.wait(**kwargs)
        
class DockerVolume(object):
    def __init__(self, volume):
        self.__volume = volume
        
    @property
    def volume(self):
        self.__volume
        
    @property
    def attributes(self):
        self.__volume.reload()
        return self.__volume.attrs
    
    def get_attribute(self, attr_path):
        names = attr_path.split('.')
        attrs = self.attributes
        res = attrs
        for i, name in enumerate(names):
            if name in res:
                res = res[name]
            else:
                raise FunctionalException(f"Attribute '{'.'.join(names[:i+1])}' doesn't exist (requested attribute: '{attr_path}' ; volume attributes: {attrs})")
        return res
    



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: docker_controler.proto
# Protobuf Python Version: 5.28.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    1,
    '',
    'docker_controler.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16\x64ocker_controler.proto\x12\x10\x64ocker_controler\"\x1d\n\x1bGetAllContainerNamesRequest\"7\n\x1cGetAllContainerNamesResponse\x12\x17\n\x0f\x63ontainer_names\x18\x01 \x03(\t\"%\n\x15StartContainerRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"2\n\x16StartContainerResponse\x12\x18\n\x10\x63ontainer_status\x18\x01 \x01(\t\"$\n\x14StopContainerRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"1\n\x15StopContainerResponse\x12\x18\n\x10\x63ontainer_status\x18\x01 \x01(\t\"\'\n\x17RestartContainerRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"4\n\x18RestartContainerResponse\x12\x18\n\x10\x63ontainer_status\x18\x01 \x01(\t2\xc9\x03\n\x16\x44ockerControlerService\x12w\n\x14GetAllContainerNames\x12-.docker_controler.GetAllContainerNamesRequest\x1a..docker_controler.GetAllContainerNamesResponse\"\x00\x12\x65\n\x0eStartContainer\x12\'.docker_controler.StartContainerRequest\x1a(.docker_controler.StartContainerResponse\"\x00\x12\x62\n\rStopContainer\x12&.docker_controler.StopContainerRequest\x1a\'.docker_controler.StopContainerResponse\"\x00\x12k\n\x10RestartContainer\x12).docker_controler.RestartContainerRequest\x1a*.docker_controler.RestartContainerResponse\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'docker_controler_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_GETALLCONTAINERNAMESREQUEST']._serialized_start=44
  _globals['_GETALLCONTAINERNAMESREQUEST']._serialized_end=73
  _globals['_GETALLCONTAINERNAMESRESPONSE']._serialized_start=75
  _globals['_GETALLCONTAINERNAMESRESPONSE']._serialized_end=130
  _globals['_STARTCONTAINERREQUEST']._serialized_start=132
  _globals['_STARTCONTAINERREQUEST']._serialized_end=169
  _globals['_STARTCONTAINERRESPONSE']._serialized_start=171
  _globals['_STARTCONTAINERRESPONSE']._serialized_end=221
  _globals['_STOPCONTAINERREQUEST']._serialized_start=223
  _globals['_STOPCONTAINERREQUEST']._serialized_end=259
  _globals['_STOPCONTAINERRESPONSE']._serialized_start=261
  _globals['_STOPCONTAINERRESPONSE']._serialized_end=310
  _globals['_RESTARTCONTAINERREQUEST']._serialized_start=312
  _globals['_RESTARTCONTAINERREQUEST']._serialized_end=351
  _globals['_RESTARTCONTAINERRESPONSE']._serialized_start=353
  _globals['_RESTARTCONTAINERRESPONSE']._serialized_end=405
  _globals['_DOCKERCONTROLERSERVICE']._serialized_start=408
  _globals['_DOCKERCONTROLERSERVICE']._serialized_end=865
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

import docker_controler_pb2 as docker__controler__pb2

GRPC_GENERATED_VERSION = '1.68.1'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + f' but the generated code in docker_controler_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class DockerControlerServiceStub(object):
    """DockerControlerService represents the querying service provided by docker controler.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetAllContainerNames = channel.unary_unary(
                '/docker_controler.DockerControlerService/GetAllContainerNames',
                request_serializer=docker__controler__pb2.GetAllContainerNamesRequest.SerializeToString,
                response_deserializer=docker__controler__pb2.GetAllContainerNamesResponse.FromString,
                _registered_method=True)
        self.StartContainer = channel.unary_unary(
                '/docker_controler.DockerControlerService/StartContainer',
                request_serializer=docker__controler__pb2.StartContainerRequest.SerializeToString,
                response_deserializer=docker__controler__pb2.StartContainerResponse.FromString,
                _registered_method=True)
        self.StopContainer = channel.unary_unary(
                '/docker_controler.DockerControlerService/StopContainer',
                request_serializer=docker__controler__pb2.StopContainerRequest.SerializeToString,
                response_deserializer=docker__controler__pb2.StopContainerResponse.FromString,
                _registered_method=True)
        self.RestartContainer = channel.unary_unary(
                '/docker_controler.DockerControlerService/RestartContainer',
                request_serializer=docker__controler__pb2.RestartContainerRequest.SerializeToString,
                response_deserializer=docker__controler__pb2.RestartContainerResponse.FromString,
                _registered_method=True)


class DockerControlerServiceServicer(object):
    """DockerControlerService represents the querying service provided by docker controler.
    """

    def GetAllContainerNames(self, request, context):
        """Get all container names.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StartContainer(self, request, context):
        """Start a container.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StopContainer(self, request, context):
        """Stop a container.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RestartContainer(self, request, context):
        """Restart a container.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_DockerControlerServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetAllContainerNames': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAllContainerNames,
                    request_deserializer=docker__controler__pb2.GetAllContainerNamesRequest.FromString,
                    response_serializer=docker__controler__pb2.GetAllContainerNamesResponse.SerializeToString,
            ),
            'StartContainer': grpc.unary_unary_rpc_method_handler(
                    servicer.StartContainer,
                    request_deserializer=docker__controler__pb2.StartContainerRequest.FromString,
                    response_serializer=docker__controler__pb2.StartContainerResponse.SerializeToString,
            ),
            'StopContainer': grpc.unary_unary_rpc_method_handler(
                    servicer.StopContainer,
                    request_deserializer=docker__controler__pb2.StopContainerRequest.FromString,
                    response_serializer=docker__controler__pb2.StopContainerResponse.SerializeToString,
            ),
            'RestartContainer': grpc.unary_unary_rpc_method_handler(
                    servicer.RestartContainer,
                    request_deserializer=docker__controler__pb2.RestartContainerRequest.FromString,
                    response_serializer=docker__controler__pb2.RestartContainerResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'docker_controler.DockerControlerService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('docker_controler.DockerControlerService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class DockerControlerService(object):
    """DockerControlerService represents the querying service provided by docker controler.
    """

    @staticmethod
    def GetAllContainerNames(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/docker_controler.DockerControlerService/GetAllContainerNames',
            docker__controler__pb2.GetAllContainerNamesRequest.SerializeToString,
            docker__controler__pb2.GetAllContainerNamesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def StartContainer(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/docker_controler.DockerControlerService/StartContainer',
            docker__controler__pb2.StartContainerRequest.SerializeToString,
            docker__controler__pb2.StartContainerResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def StopContainer(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/docker_controler.DockerControlerService/StopContainer',
            docker__controler__pb2.StopContainerRequest.SerializeToString,
            docker__controler__pb2.StopContainerResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def RestartContainer(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/docker_controler.DockerControlerService/RestartContainer',
            docker__controler__pb2.RestartContainerRequest.SerializeToString,
            docker__controler__pb2.RestartContainerResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)




#################################################
# HolAdo (Holistic Automation do)
#
# (C) Copyright 2021-2025 by Eric Klumpp
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

# The Software is provided “as is”, without warranty of any kind, express or implied, including but not limited to the warranties of merchantability, fitness for a particular purpose and noninfringement. In no event shall the authors or copyright holders be liable for any claim, damages or other liability, whether in an action of contract, tort or otherwise, arising from, out of or in connection with the software or the use or other dealings in the Software.
#################################################

import logging
import json
from holado_core.common.resource.persisted_data_manager import PersistedDataManager
from holado_python.standard_library.typing import Typing
from holado_core.common.exceptions.technical_exception import TechnicalException
from holado_core.common.tools.tools import Tools

logger = logging.getLogger(__name__)



class PersistedMethodToCallManager(PersistedDataManager):
    def __init__(self, data_name="method", table_name="method_to_call", db_name="default"):
        super().__init__(data_name=data_name, table_name=table_name, 
                         table_sql_create=self._get_default_table_sql_create(table_name), 
                         db_name=db_name)
    
    def initialize(self, resource_manager, expression_evaluator):
        super().initialize(resource_manager)
        self.__expression_evaluator = expression_evaluator
    
    def _get_default_table_sql_create(self, table_name):
        return f"""CREATE TABLE {table_name} (
                id INTEGER PRIMARY KEY,
                function_qualname text NOT NULL,
                self_getter text,
                args text,
                kwargs text,
                use text NOT NULL,
                use_index integer NOT NULL
            )"""
    
    def add_function_to_call(self, function_qualname, args_list=None, kwargs_dict=None, use="default", use_index=0):
        """Add a function to call.
        @param function_qualname: Qualified name of function
        @param args_list: List of function args (default: None)
        @param kwargs_dict: Dict of function kwargs (default: None)
        @param use: Define persistent usage. It usually corresponds to a specific scope.
        @param use_index: use index, useable to order the functions to call. By default all are 0. If set to None, it is automatically set to max(use_index)+1.
        """
        self.add_method_to_call(function_qualname, None, args_list, kwargs_dict, use, use_index)
    
    def add_method_to_call(self, function_qualname, self_getter_eval_str, args_list=None, kwargs_dict=None, use="default", use_index=0):
        """Add a method to call.
        @param function_qualname: Qualified name of function
        @param self_getter_eval_str: String to eval in order to get the self instance to use when calling method 
        @param args_list: List of function args (default: None)
        @param kwargs_dict: Dict of function kwargs (default: None)
        @param use: Define persistent usage. It usually corresponds to a specific scope.
        @param use_index: use index, useable to order the functions to call. By default all are 0. If set to None, it is automatically set to max(use_index)+1.
        """
        if use_index is None:
            use_index = self.__get_use_next_index(use)
            
        data = {
            'function_qualname': function_qualname,
            'self_getter': self_getter_eval_str,
            'use': use,
            'use_index': use_index
            }
        if args_list is not None:
            data['args'] = json.dumps(args_list)
        if kwargs_dict is not None:
            data['kwargs'] = json.dumps(kwargs_dict)
            
        self.add_persisted_data(data)
    
    def __get_use_next_index(self, use):
        datas = self.get_persisted_datas({'use':use})
        if datas:
            return max(map(lambda x:x['a'], datas)) + 1
        else:
            return 0
    
    def call_functions_and_methods(self, use="default", use_index=None, delete_after_call=False):
        """Call methods of given use
        @param use: Define persistent usage. It usually corresponds to a specific scope.
        @param use_index: If defined, call only functions and methods of given index.
        @param delete_after_call: Define if function or method is deleted after call from persisted data.
        """
        # Get functions and methods to call
        filter_data = {'use':use}
        if use_index is not None:
            filter_data['use_index'] = use_index
        methods_data = self.get_persisted_datas(filter_data)
        
        # Call methods
        if methods_data:
            for meth_index, meth_data in enumerate(methods_data):
                do_delete = delete_after_call
                try:
                    self._call_function_or_method(meth_data)
                except Exception as exc:
                    msg_list = [f"Error while calling following method (use: '{use}' ; use index: {use_index} ; method index: {meth_index} ; delete after call: {delete_after_call}):"]
                    msg_list.append(Tools.represent_object(meth_data, 8))
                    msg_list.append("    Error:")
                    msg_list.append(Tools.represent_exception(exc, indent=8))
                    msg_list.append("  => Continue to process persisted methods")
                    msg_list.append("     WARNING: this method is removed from persisted methods to avoid recursive and blocking errors")
                    logger.error("\n".join(msg_list))
                    do_delete = True
                
                if do_delete:
                    self.__delete_function_or_method(meth_data)
    
    def __delete_function_or_method(self, function_or_method_data):
        filter_data = {'id':function_or_method_data['id']}
        self.delete_persisted_data(filter_data)
    
    def _call_function_or_method(self, function_or_method_data):
        _, func = self.__expression_evaluator.evaluate_python_expression(function_or_method_data['function_qualname'])
        if not Typing.is_function(func):
            raise TechnicalException(f"Failed to evaluate python expression '{function_or_method_data['function_qualname']}' as a function (obtained: {func} [type: {Typing.get_object_class_fullname(func)}] ; function data: {function_or_method_data})")
        
        func_self = None
        if function_or_method_data['self_getter'] is not None:
            _, func_self = self.__expression_evaluator.evaluate_python_expression(function_or_method_data['self_getter'])
        
        if function_or_method_data['args'] is not None:
            args = json.loads(function_or_method_data['args'])
        else:
            args = []
        if function_or_method_data['kwargs'] is not None:
            kwargs = json.loads(function_or_method_data['kwargs'])
        else:
            kwargs = {}
        
        if func_self is not None:
            func(func_self, *args, **kwargs)
        else:
            func(*args, **kwargs)
        
    




#################################################
# HolAdo (Holistic Automation do)
#
# (C) Copyright 2021-2025 by Eric Klumpp
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

# The Software is provided “as is”, without warranty of any kind, express or implied, including but not limited to the warranties of merchantability, fitness for a particular purpose and noninfringement. In no event shall the authors or copyright holders be liable for any claim, damages or other liability, whether in an action of contract, tort or otherwise, arising from, out of or in connection with the software or the use or other dealings in the Software.
#################################################

from holado.holado_config import Config
import logging
import multiprocessing
import time
from holado.common.context.session_context import SessionContext
from holado_multitask.multitasking.multitask_manager import MultitaskManager
from holado_python.standard_library.typing import Typing
from holado.common.handlers.undefined import undefined_argument
from holado_core.common.exceptions.technical_exception import TimeoutTechnicalException
from holado_core.common.exceptions.functional_exception import FunctionalException

logger = logging.getLogger(__name__)


class Process(multiprocessing.context.Process):
    """ Helper class managing a multiprocessing process.
    WARNING: Do not use, it is under construction
    """
    
    def __init__(self, name, default_wait_timeout = None, register_process = True, delay_before_run_sec=None):
        super().__init__()
        
        if default_wait_timeout is None:
            default_wait_timeout = Config.timeout_seconds * 1000
            
        self.__name = name
        self.__unique_name = SessionContext.instance().multitask_manager.prepare_process(name)
        self.__default_wait_timeout = default_wait_timeout
        self.__delay_before_run_sec = delay_before_run_sec

        # Put process as daemon
        self.daemon = True
        
        # Manage process status
        # self._is_lock = threading.Lock()
        # self._is_idle = threading.Event()
        # self._is_idle.set()
        
        # Register process
        if register_process:
            # Registered name has to be unique
            SessionContext.instance().processes_manager.register_process(self.__unique_name, self)
        
    @property
    def name(self):
        return self.__name
    
    @property
    def unique_name(self):
        return self.__unique_name
    
    @property
    def default_wait_timeout(self):
        return self.__default_wait_timeout
        
    @default_wait_timeout.setter
    def default_wait_timeout(self, timeout):
        self.__default_wait_timeout = timeout
        
    @property
    def delay_before_run_seconds(self):
        return self.__delay_before_run_sec
        
    def start(self)->None:
        super().start()
        SessionContext.instance().multitask_manager.set_process_id(self.__unique_name, MultitaskManager.get_process_id(process=self))
    
    def run(self)->None:
        SessionContext.instance().multitask_manager.set_process_thread_uid(self.__unique_name, MultitaskManager.get_thread_uid())
        super().run()
    
    def join(self, timeout=undefined_argument, raise_if_still_alive=True):
        """
        See Process.join documentation for default behavior.
        
        This implementation tries to avoid deadlocks by special and default timeout value undefined_argument.
        With any other timeout value, including None, default behavior is happening.
        If timeout argument is undefined_argument, it means that it should stop by itself without deadlock.
        In this case, a timeout value Config.join_timeout_seconds is used.
        
        If argument raise_if_still_alive is True (default) and if process is still alive after join, an exception is raised.
        If timeout is undefined_argument, a TimeoutTechnicalException is raised, else a FunctionalException.
        """
        timeout_str = "long and finite" if timeout is undefined_argument else f"{timeout}s" if timeout else "none"
        logger.debug(f"Joining process '{self.name}' ; timeout={timeout_str} ...")
        
        real_timeout = timeout if timeout is not undefined_argument else Config.join_timeout_seconds
        super().join(real_timeout)
        
        logger.debug(f"End of joining process '{self.name}' ; timeout={timeout_str} ; alive:{self.is_alive()}")
        
        if self.is_alive():
            if raise_if_still_alive:
                if timeout is undefined_argument:
                    raise TimeoutTechnicalException(f"Process '{self.name}' is still alive after join")
                else:
                    raise FunctionalException(f"Process '{self.name}' is still alive after join")
            else:
                logger.error(f"Process '{self.name}' is still alive after join")
    
    def wait_before_run(self):
        if self.delay_before_run_seconds is not None:
            try:
                time.sleep(self.delay_before_run_seconds)
            except Exception as exc:
                logger.exception(f"Exception while waiting delay before run (delay: {self.delay_before_run_seconds} ; type: {Typing.get_object_class_fullname(self.delay_before_run_seconds)})")
                
    # def wait_is_idle(self, timeout=None):
    #     """ Wait event self._is_idle"""
    #     self._wait(self.__is_idle, "is idle", timeout=timeout)
    #
    # def _wait(self, event, event_details, timeout=None):
    #     """ Wait given threading.Event"""
    #     is_set = False
    #     wait_round_counter = 0
    #     round_wait_seconds = 1
    #     if timeout is None:
    #         timeout = self.__default_wait_timeout
    #
    #     logger.debug("[{}] Waiting {}...".format(self.name, event_details))
    #     while not is_set:
    #         wait_round_counter += 1
    #         is_set = event.wait(round_wait_seconds)
    #
    #         if not is_set:
    #             if wait_round_counter * round_wait_seconds * 1000 > timeout:
    #                 raise TimeoutError("[{}] Timeout when waiting {}".format(self.name, event_details))
    #             elif wait_round_counter % 10 == 0:
    #                 logger.debug("[{}] Still waiting {} (round {})".format(self.name, event_details, wait_round_counter))
    #     logger.debug("[{}] Finished waiting {}...".format(self.name, event_details))

class InterruptableProcess(Process):
    """ Helper class managing a process interruptable.
    It gives only the interface, sub-classes must implement if it is really interruptable and how.
    """
    
    def __init__(self, name, default_wait_timeout = None, register_process = True, delay_before_run_sec=None):
        super().__init__(name, default_wait_timeout = default_wait_timeout, register_process = register_process, delay_before_run_sec=delay_before_run_sec)

        # Manage interrupt
        # self.__interrupt_lock = threading.Lock()
        # self.__interrupt_event = threading.Event()
    
    @property    
    def is_interruptable(self):
        # return True
        raise NotImplementedError
    
    @property
    def is_interrupted(self):
        return False
    #     with self.__interrupt_lock:
    #         return self.__interrupt_event.is_set()

    def interrupt(self):
        # if not self.is_interrupted:
        #     logger.debug("Interrupt thread '{}'".format(self.name))
        #     with self.__interrupt_lock:
        #         self.__interrupt_event.set()
        raise NotImplementedError
    
    def _raise_if_interrupted(self):
        if self.is_interrupted:
            raise InterruptedError
        


class ThreadAsProcess(InterruptableProcess):
    """
    WARNING: Do not use, it is under construction
    """
    def __init__(self, thread):
        super().__init__()
        self.__thread = thread
        
    def run(self):
        # Start running by superclass run
        super().run()
        
        return self.__thread.run()
    
    





#################################################
# HolAdo (Holistic Automation do)
#
# (C) Copyright 2021-2025 by Eric Klumpp
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

# The Software is provided “as is”, without warranty of any kind, express or implied, including but not limited to the warranties of merchantability, fitness for a particular purpose and noninfringement. In no event shall the authors or copyright holders be liable for any claim, damages or other liability, whether in an action of contract, tort or otherwise, arising from, out of or in connection with the software or the use or other dealings in the Software.
#################################################

from holado_core.common.exceptions.functional_exception import FunctionalException
from holado_core.common.exceptions.technical_exception import TechnicalException
import logging
from holado_core.common.tools.tools import Tools
import json
from holado_core.common.tools.converters.converter import Converter
from holado_json.ipc.json_converter import JsonConverter

logger = logging.getLogger(__name__)

try:
    import requests
    from requests.auth import HTTPBasicAuth
    with_requests = True
except Exception as exc:
    if Tools.do_log(logger, logging.DEBUG):
        logger.debug(f"RestClient is not available. Initialization failed on error: {exc}")
    with_requests = False


class RestClient(object):
    """
    REST client
    """
    
    @classmethod
    def is_available(cls):
        return with_requests
    
    def __init__(self, name, url, headers=None):
        self.__name = name
        self.__url = url.rstrip('/')
        self.__headers = headers if headers is not None else {}
        self.__kwargs = {}
    
    @property
    def name(self):
        return self.__name
    
    def authenticate_by_user(self, user, pwd):
        if Tools.do_log(logger, logging.DEBUG):
            logger.debug(f"[{self.name}] Authenticate with user ({user},{pwd})")
        self.__kwargs['auth'] = HTTPBasicAuth(user, pwd)
    
    def authenticate_by_token(self, token_type, access_token):
        if Tools.do_log(logger, logging.DEBUG):
            logger.debug(f"[{self.name}] Authenticate with token type '{token_type}' and access token '{access_token}'")
        self.__headers['Authorization'] = f'{token_type} {access_token}'
    
    def with_ssl(self, ssl_kwargs):
        unexpected_keys = set(ssl_kwargs.keys()).difference(['verify', 'cert'])
        if len(unexpected_keys) > 0:
            raise TechnicalException(f"Unmanaged SSL arguments: {unexpected_keys}")
        
        self.__kwargs.update(ssl_kwargs)
        
    def response_result(self, response, status_ok=200):
        if "application/json" in response.headers['Content-Type']:
            res = response.json()
        elif response.headers['Content-Type'].startswith('text'):
            res = response.text
        else:
            res = response.content
        
        is_ok = isinstance(status_ok, list) and response.status_code in status_ok or response.status_code == status_ok
        if not is_ok:
            raise FunctionalException(f"[{self.name}] Request failed with status {response.status_code} (expected success status: {status_ok}) on error: {res}")
        
        return res
    
    def request(self, method, path, **kwargs):
        url = self.__build_url(path=path)
        r_kwargs = self.__build_kwargs(**kwargs)
        
        if Tools.do_log(logger, logging.DEBUG):
            logger.debug(f"[{self.name}] Calling {method} on url '{url}' with arguments {r_kwargs}")
        try:
            res = requests.request(method, url, **r_kwargs)
        except Exception as exc:
            raise TechnicalException(f"[{self.name}] Failed to process {method} '{url}' with arguments {r_kwargs}") from exc
        
        if Tools.do_log(logger, logging.DEBUG):
            logger.debug(f"[{self.name}] {method} on url '{url}' => {res}")
        return res
        
    def delete(self, path, data=None, **kwargs):
        return self.request("DELETE", path, data=data, **kwargs)
        
    def get(self, path, data=None, **kwargs):
        return self.request("GET", path, data=data, **kwargs)
        
    def patch(self, path, data=None, **kwargs):
        return self.request("PATCH", path, data=data, **kwargs)
        
    def post(self, path, data=None, json=None, **kwargs):
        return self.request("POST", path, data=data, json=json, **kwargs)
        
    def put(self, path, data=None, **kwargs):
        return self.request("PUT", path, data=data, **kwargs)
        
    def __build_url(self, path):
        return f"{self.__url}/{path.lstrip('/')}" 
        
    def __build_kwargs(self, **request_kwargs):
        res = dict(self.__kwargs)
        if self.__headers:
            res['headers'] = self.__headers
        if request_kwargs:
            # Add all kwargs except data
            data = request_kwargs.pop('data', None)
            for key in list(request_kwargs.keys()):
                if key in res and isinstance(res[key], dict):
                    res[key].update(request_kwargs.pop(key))
            res.update(request_kwargs)
            
            # Ensure data is in right format
            if data is not None:
                # Convert data to json object for types not supported by requests.request
                if not (isinstance(data, str) or Converter.is_dict(data) or Converter.is_list(data) or isinstance(data, bytes) or Converter.is_file_like(data)):
                    converter = JsonConverter()
                    data = converter.to_json(data)
                
                # Convert data to string for some content types
                if not isinstance(data, str):
                    content_type = res['headers']['Content-Type'] if 'headers' in res and 'Content-Type' in res['headers'] else None
                    if content_type is not None and 'application/json' in content_type:
                        data = json.dumps(data)
                res['data'] = data
            
        return res 
        
        




#################################################
# HolAdo (Holistic Automation do)
#
# (C) Copyright 2021-2025 by Eric Klumpp
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

# The Software is provided “as is”, without warranty of any kind, express or implied, including but not limited to the warranties of merchantability, fitness for a particular purpose and noninfringement. In no event shall the authors or copyright holders be liable for any claim, damages or other liability, whether in an action of contract, tort or otherwise, arising from, out of or in connection with the software or the use or other dealings in the Software.
#################################################

import logging
from holado_report.report.builders.report_builder import ReportBuilder
from holado_python.common.tools.datetime import DateTime,\
    FORMAT_DATETIME_HUMAN_SECOND

logger = logging.getLogger(__name__)



class SummaryScenarioReportBuilder(ReportBuilder):
    def __init__(self, filepath):
        self.__filepath = filepath
        self.__file = None
        
    def build(self):
        '''
        The file is built after each scenario
        '''
        pass
        
    def after_scenario(self, scenario, scenario_report=None):
        from holado_report.report.report_manager import ReportManager
        status_validation, step_failed, step_number = ReportManager.get_current_scenario_status_information(scenario)
        
        if status_validation == "Passed":
            self.__file_add_scenario_success(scenario, status_validation)
        else:
            self.__file_add_scenario_fail(scenario, status_validation, step_failed, step_number)
            
    def after_all(self):
        # Manage file fail
        if self.__file is not None:
            self.__file.close()
            self.__file = None
        
    def __file_add_scenario_success(self, scenario, status_validation):
        self.__open_file_if_needed()
        self.__file_write(scenario, None, status_validation)
        self.__file.flush()
        
    def __file_add_scenario_fail(self, scenario, status_validation, step_failed, step_number):
        self.__open_file_if_needed()
        if step_failed is not None:
            self.__file_write(scenario, f"step {step_number} (l.{step_failed.line})", status_validation)
        else:
            self.__file_write(scenario, f"step ? (missing step implementation ?)", status_validation)
        self.__file.flush()
    
    def __file_write(self, scenario, text, status_validation):
        dt = DateTime.now()
        dt_str = DateTime.datetime_2_str(dt, FORMAT_DATETIME_HUMAN_SECOND)
        
        if text:
            self.__file.write(f"{dt_str} - {scenario.filename} at l.{scenario.line} - {text} - {status_validation}\n")
        else:
            self.__file.write(f"{dt_str} - {scenario.filename} at l.{scenario.line} - {status_validation}\n")
    
    def __open_file_if_needed(self):
        if self.__file is None:
            self.__file = open(self.__filepath, "wt")
    



import os
import sys
import shutil

here = os.path.abspath(os.path.dirname(__file__))

holado_src_path = os.path.normpath(os.path.join(here, "..", "..", "..", ".."))
holado_path = os.path.normpath(os.path.join(holado_src_path, ".."))
sys.path.insert(0, holado_src_path)
print(f"Inserted path: {holado_src_path}")

from holado_core.common.tools.path_manager import PathManager
# from holado_protobuf.ipc.protobuf.protobuf_compiler import ProtobufCompiler
from holado_grpc.ipc.rpc.grpc_compiler import GRpcCompiler

# def compile_protobuf_proto(remove_destination=True):
#     # Define protoc path
#     protoc_exe_path = os.path.join(holado_path, "dependencies", "protoc", "protoc-23.4-linux-x86_64", "bin", "protoc")
#
#     # Define proto and generated paths
#     proto_path = os.path.join(here, "definitions")
#     destination_path = os.path.join(here, "generated")
#
#     # Remove existing destination
#     if remove_destination and os.path.exists(destination_path) and os.path.isdir(destination_path):
#         shutil.rmtree(destination_path)
#
#     protoc = ProtobufCompiler()
#     protoc.protoc_exe_path = protoc_exe_path
#
#     protoc.register_proto_path(os.path.join(proto_path, "xxx"), os.path.join(destination_path, "xxx"), os.path.join(proto_path, "xxx", "yyy"))
#
#     protoc.compile_all_proto()

def compile_grpc_proto(remove_destination=True):
    # Define proto and generated paths
    proto_path = os.path.join(here, "definitions")
    destination_path = os.path.join(here, "generated")
    
    # Remove existing destination
    if remove_destination and os.path.exists(destination_path) and os.path.isdir(destination_path):
        if destination_path != proto_path:
            shutil.rmtree(destination_path)
        else:
            path_manager = PathManager()
            glob_pattern = os.path.join(destination_path, "*.py")
            path_manager.remove_paths(glob_pattern)
    
    protoc = GRpcCompiler()
    
    protoc.register_proto_path(proto_path, destination_path)
    
    protoc.compile_all_proto()

if __name__ == "__main__":
    # compile_protobuf_proto()
    # compile_grpc_proto(remove_destination=False)
    compile_grpc_proto()
    
    




#################################################
# HolAdo (Holistic Automation do)
#
# (C) Copyright 2021-2025 by Eric Klumpp
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

# The Software is provided “as is”, without warranty of any kind, express or implied, including but not limited to the warranties of merchantability, fitness for a particular purpose and noninfringement. In no event shall the authors or copyright holders be liable for any claim, damages or other liability, whether in an action of contract, tort or otherwise, arising from, out of or in connection with the software or the use or other dealings in the Software.
#################################################

import logging
from holado_report.report.builders.report_builder import ReportBuilder
from holado_core.common.tools.tools import Tools
from holado_core.common.exceptions.technical_exception import TechnicalException
import json

logger = logging.getLogger(__name__)



class FailureReportBuilder(ReportBuilder):
    """Failure report builder
    Supported formats: 'txt', 'json', 'xml'
    """
    def __init__(self, filepath, file_format='xml'):
        if file_format not in ['txt', 'json', 'xml']:
            raise TechnicalException(f"Unmanaged format '{file_format}' (possible formats: 'txt', 'json')")
        
        self.__filepath = filepath
        self.__file_format = file_format
        self.__failures = {}
        
    def build(self):
        '''
        The file is built after each scenario
        '''
        pass
        
    def after_scenario(self, scenario, scenario_report=None):
        from holado_report.report.report_manager import ReportManager
        status_validation, step_failed, step_number = ReportManager.get_current_scenario_status_information(scenario)
        
        if status_validation != "Passed":
            step_error_message = ReportManager.get_step_error_message(step_failed).strip()
            if step_error_message not in self.__failures:
                self.__failures[step_error_message] = []
            
            if self.__file_format == 'txt':
                msg_list = []
                msg_list.append(f"scenario in {scenario.filename} at l.{scenario.line} - step {step_number} (l.{step_failed.line}) - {status_validation}")
                msg_list.append(f"    Feature/Scenario: {scenario.feature.name}  =>  {scenario.name}")
                msg_list.append(f"    Report: {scenario_report.report_path}")
                msg_list.append(f"    Tags: -t " + " -t ".join(scenario.feature.tags + scenario.tags))
                msg_scenario = "\n".join(msg_list)
                
                self.__failures[step_error_message].append(msg_scenario)
            elif self.__file_format in ['json', 'xml']:
                scenario_info = {
                    'title': f"{scenario.filename} - l.{scenario.line} - step {step_number} (l.{step_failed.line}) - {status_validation}",
                    'scenario': f"{scenario.feature.name}  =>  {scenario.name}",
                    'report': scenario_report.report_path,
                    'tags': "-t " + " -t ".join(scenario.feature.tags + scenario.tags),
                    }
                self.__failures[step_error_message].append(scenario_info)
            else:
                raise TechnicalException(f"Unmanaged format '{self.__file_format}' (possible formats: 'txt', 'json')")
            
            self.__update_file()
    
    def __update_file(self):
        with open(self.__filepath, "wt") as fout:
            if self.__file_format == 'txt':
                for failure, scenarios_messages in self.__failures.items():
                    fout.write(failure + "\n")
                    fout.write("\n")
                    for msg in scenarios_messages:
                        fout.write(Tools.indent_string(4, msg) + "\n")
                        fout.write("\n")
            elif self.__file_format == 'json':
                json_str = json.dumps(self.__failures, ensure_ascii=False, indent=4)
                fout.write(json_str)
            elif self.__file_format == 'xml':
                self.__file_write_failures(fout, self.__failures, sort_by_nb_scenario=True)
            else:
                raise TechnicalException(f"Unmanaged format '{self.__file_format}' (possible formats: 'txt', 'json')")

    def __file_write_failures(self, fout, __failures, sort_by_nb_scenario=True):
        fout.write("<failures>\n")
        
        if sort_by_nb_scenario:
            for error_message, scenarios in sorted(self.__failures.items(), key=lambda item:-len(item[1])):
                self.__file_write_failure(fout, error_message, scenarios)
        else:
            for error_message, scenarios in self.__failures.items():
                self.__file_write_failure(fout, error_message, scenarios)
    
        fout.write("</failures>\n")
    
    def __file_write_failure(self, fout, error_message, scenarios):
        msg_list = []
        
        msg_list.append("    <failure>")
        
        if "\n" in error_message:
            msg_list.append(f"        <error_message>")
            msg_list.append(Tools.indent_string(12, error_message))
            msg_list.append(f"        </error_message>")
        else:
            msg_list.append(f"        <error_message>{error_message}</error_message>")
        
        msg_list.append(f"        <scenarios>")
        for scenario_info in scenarios:
            msg_list.append(f"            <scenario>")
            for key, value in scenario_info.items():
                msg_list.append(f"                <{key}>{value}</{key}>")
            msg_list.append(f"            </scenario>")
        msg_list.append(f"        </scenarios>")
        
        msg_list.append("    </failure>")
        
        msg = "\n".join(msg_list) + "\n"
        fout.write(msg)
    







#################################################
# HolAdo (Holistic Automation do)
#
# (C) Copyright 2021-2025 by Eric Klumpp
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

# The Software is provided “as is”, without warranty of any kind, express or implied, including but not limited to the warranties of merchantability, fitness for a particular purpose and noninfringement. In no event shall the authors or copyright holders be liable for any claim, damages or other liability, whether in an action of contract, tort or otherwise, arising from, out of or in connection with the software or the use or other dealings in the Software.
#################################################

import logging
from holado_core.common.tools.tools import Tools
from holado.common.context.session_context import SessionContext
from holado_core.common.exceptions.functional_exception import FunctionalException
from yaml.loader import BaseLoader, FullLoader, SafeLoader


logger = logging.getLogger(__name__)

try:
    import yaml
    with_yaml = True
except Exception as exc:
    if Tools.do_log(logger, logging.DEBUG):
        logger.debug(f"YAML is not available. Initialization failed on error: {exc}")
    with_yaml = False


class YAMLManager(object):
    """
    Manage actions on YAML files.
    """
    
    @classmethod
    def is_available(cls):
        return with_yaml
    
    @classmethod
    def __get_path_manager(cls):
        return SessionContext.instance().path_manager
    
    @classmethod
    def load_file(cls, file_path, with_only_strings=False, with_full_yaml_features=False):
        if with_only_strings and with_full_yaml_features:
            raise FunctionalException(f"It is not possible to set both with_only_strings and with_full_yaml_features to True")
        
        if with_only_strings:
            loader = BaseLoader
        elif with_full_yaml_features:
            loader = FullLoader
        else:
            loader = SafeLoader
        
        with open(file_path, 'r') as file:
            res = yaml.load(file, Loader=loader)
        
        return res
    
    @classmethod
    def load_multiple_documents_file(cls, file_path, with_only_strings=False, with_full_yaml_features=False):
        if with_only_strings and with_full_yaml_features:
            raise FunctionalException(f"It is not possible to set both with_only_strings and with_full_yaml_features to True")
        
        if with_only_strings:
            loader = BaseLoader
        elif with_full_yaml_features:
            loader = FullLoader
        else:
            loader = SafeLoader
        
        with open(file_path, 'r') as file:
            res = list(yaml.load_all(file, Loader=loader))
        
        return res
    
    @classmethod
    def save_file(cls, data, file_path):
        cls.__get_path_manager().makedirs(file_path)
        with open(file_path, 'w') as file:
            yaml.dump(data, file)



